/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "definitions.h"
#include "utils.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>



void match_string_length (char **dest, int len)
{
	int dlen = g_utf8_strlen(*dest, -1);
	if (dlen < len)
	{
		char  *sfill = g_strnfill(len-dlen, ' ');
		char *new_str = g_strconcat(*dest, sfill, NULL);
		g_free(*dest);
		g_free(sfill);
		*dest = new_str;
	}else if (dlen > len)
	{
		g_strchomp(*dest + len);	
	}
}

char *get_bytes_string_base (unsigned long long nbytes, unsigned long long base,
							 const char *format, const char *formatd)
{
	char *str;
	unsigned long long decimalv; 
	decimalv = (nbytes < base * 10) ? (nbytes % base) * 10 / base : 0;
	if (decimalv == 0)
		str = g_strdup_printf(format, nbytes / base);
	else
		str = g_strdup_printf(formatd, nbytes / base, decimalv);
	return str;
}

char* get_bytes_string (unsigned long long nbytes)
{
	const unsigned long long ctKB = 1024, ctMB = ctKB*1024, 
		ctGB = ctMB*1024, ctTB = ctGB*1024;
	char *str;
	if (nbytes < ctKB)
		str = g_strdup_printf(_("%Lu B"), nbytes);
	else if (nbytes < ctMB)
		str = get_bytes_string_base(nbytes, ctKB, _("%Lu KB"), _("%Lu.%Lu KB"));
	else if (nbytes < ctGB)
		str = get_bytes_string_base(nbytes, ctMB, _("%Lu MB"), _("%Lu.%Lu MB"));
	else if (nbytes < ctTB)
		str = get_bytes_string_base(nbytes, ctGB, _("%Lu GB"), _("%Lu.%Lu GB"));
	else
		str = get_bytes_string_base(nbytes, ctTB, _("%Lu TB"), _("%Lu.%Lu TB"));
	return str;
}


gboolean drop_effective_user_to_sudo ()
{
	gboolean dropped = FALSE;
	if (getuid() == 0)
	{
		const char *sudo_uid = g_getenv("SUDO_UID");
		const char *sudo_gid = g_getenv("SUDO_GID");
		
		if (sudo_uid != NULL)
		{
			seteuid(atol(sudo_uid));
			dropped = TRUE;
			if (sudo_gid != NULL)
				setegid(atol(sudo_gid));	
		}
	}
	return dropped;
}

void restore_effective_user ()
{
	seteuid(getuid());
	setegid(getgid());
}

const char* get_effective_home_dir ()
{
	struct passwd *pw;
	pw = getpwuid(geteuid());
	if (pw != NULL && pw->pw_dir!=NULL)
		return pw->pw_dir;
	
	{
		const char *home_dir = g_getenv("HOME");
		if (home_dir != NULL)
			return home_dir;
	}
	return g_get_home_dir(); /*In the current implementation(2.14) returns home for uid*/
}

void set_clipboard_text (const char *text)
{
	GdkAtom atom;
	GtkClipboard *clipboard;
	
	atom = gdk_atom_intern("CLIPBOARD", FALSE);
	clipboard = gtk_clipboard_get(atom);
	
	gtk_clipboard_set_text(clipboard, text, -1);
}

void EnsureStringLen (char **str, int *maxSize, int len)
{
	int maxl = *maxSize;
	g_assert(str != NULL && *str != NULL && maxl > 0);
	if (len < maxl)
		return;
	while (len >= maxl)
		maxl *= 2;
	*str = (char*)g_realloc(*str, maxl);
	*maxSize = maxl;
}

char *string_replace (const char* str, const char * from, const char* to)
{
	g_assert(str != NULL && from != NULL && to != NULL);
	int len = strlen(str), fromlen = strlen(from), tolen = strlen(to), maxreslen, i, j, k;
	g_assert(fromlen > 0);
	maxreslen = (len*tolen)/fromlen + 1;
	if (maxreslen < len)
		maxreslen = len;
	char *res = (char*)g_malloc0(maxreslen + 1);

	for(i = 0, j = 0; i < len;)
	{
		k = 0;
		if (i < len-fromlen+1)
		{
			for(k=0; k<fromlen; k++)
				if (str[i+k] != from[k])
					break;
		}
		
		if (k == fromlen)
		{
			for(k=0; k<tolen; k++)
			{
				g_assert(j<maxreslen);
				res[j++] = to[k];
			}
			i+=fromlen;
		}else
		{
			g_assert(j<maxreslen);
			res[j++] = str[i++];			
		}
	}
	res[j] = '\0';
	return res;
}


const char *get_file_extension (const char *fileName)
{
	g_assert(fileName != NULL);
	const char *extension = strrchr(fileName, '.');
	if (extension != NULL && extension != fileName)
		extension++;
	else
		extension = fileName + strlen(fileName);
	return extension;
}


